module MnoEnterprise
  class Nimbus::V4::EventsController < Nimbus::V4::BaseController

    private
      def hash_for(resource)
        resource
      end

      def resource_klass
        MnoEnterprise::Invoice
      end

      # Retrieve all resources
      def get_resources
        app_instances_by_uid = MnoEnterprise::AppInstance.all.index_by(&:uid)
        invoices_request = MnoEnterprise::Invoice.all
        invoices_request.params[:currency] = Settings.westcon.api.events.currency
        invoices_request.to_a.select { |invoice| invoice[:updated_at] != nil }.map { |invoice|
          invoice.billing_summary.each_with_index.map { |bill, i|
            # price_tag is deprecated
            bill['lines'].each_with_index.map { |line, j|
              price = line[:converted_price] || line[:price] || Monetize.parse(line[:price_tag])
              app_instance = app_instances_by_uid[bill[:app_uid]]
              if app_instance
                product = app_instance.app.nid
              end
              {
                  id: "evt-#{invoice.id}b#{i}l#{j}",
                  event_type: 'financial',
                  amount_cents: price.cents,
                  amount_currency: price.currency.iso_code,
                  label: bill[:label] +' - ' + line[:label],
                  tx_type: 'charge',
                  organization_id: invoice.organization.uid,
                  type: bill[:app_uid] ? 'app' : 'other',
                  app_instance_id: bill[:app_uid],
                  app_instance_product: product,
                  started_at: invoice.started_at,
                  ended_at: invoice.ended_at,
                  created_at: invoice.created_at

              }
            }.flatten
          }
        }.flatten
      end

      def get_resource(id)
        get_resources.find { |e| e[:id] == id }
      end
  end
end
