module MnoEnterprise
  class Nimbus::V4::UsersController < Nimbus::V4::BaseController

    # POST /nimbus/v4/organization/:organization_id/users
    # Create new resource and return representation
    # --
    # Override parent
    # Attempt to fetch user via email before creation
    def create
      unless params[:data]
        render_error VALIDATION_ERROR, 'Body does not contain the field: data'
        return false
      end
      attributes = params[:data].select { |k, v| params_create_whitelist.include?(k.to_sym) }
      email = attributes[:email]
      user = MnoEnterprise::User.find_by(email: email)
      if user
        render_error VALIDATION_ERROR, "User [#{user.uid}] already exist with email: #{email}"
        return
      end


      @resource ||= build_resource(attributes)

      # Save resource or set error code and status
      unless @resource.save
        render_error VALIDATION_ERROR, @resource.errors
        return
      end

      after_create_resource(@resource, attributes)

      render_response(@resource)
    end

    private
      def resource_klass
        MnoEnterprise::User
      end

      # Define which attributes can be edited during creation
      def params_create_whitelist
        [:name, :surname, :email, :role]
      end

      # Retrieve all user
      def get_resources
        parent.users
      end

      # Retrieve a user via uid
      def get_resource(uid)
        MnoEnterprise::User.find_by(uid: uid)
      end

      # Build a resource for creation
      def build_resource(attributes)
        attrs = attributes.select { |k, v| params_create_whitelist.include?(k.to_sym) }
        res = MnoEnterprise::User.new attrs
        res.password = Devise.friendly_token.first(15)
        res
      end

      # Update a resource
      def update_resource(resource, attributes)
        super(resource, attributes.reject { |k, v| k.to_sym == :role })
      end

      # Add user to the parent once created
      def after_create_resource(resource, opts = {})
        resource.free_trial_end_at = 15.years.from_now
        # Add user to organization
        parent.add_user resource
        resource.save
      end


      # Resource JSON view
      def hash_for(resource)
        hash_for_user resource
      end
  end
end
